# Dataframes
# Create a data frame. to create a dataframe, we can use the function data.frame()
cheese <- data.frame(
  1:5,
  c(FALSE, FALSE, TRUE, FALSE, FALSE),
  as.factor(c("cow", "ewe", "cow", "goat", "soy")),
  c(4, 5, 3, 3, 1)
)
# displaying the dataframe, we can notice that the columns names are not that great
cheese
# correcting the column names
colnames(cheese) <- c("Id", "Cooked", "milk", "strength")

# Or we can name the columns while creating the dataframe
cheese2 <- data.frame(
  "Id" = 1:5,
  "Cooked" = c(FALSE, FALSE, TRUE, FALSE, FALSE),
  "milk" = as.factor(c("cow", "ewe", "cow", "goat", "soy")),
  "strength" = c(4, 5, 3, 3, 1)
)
# Always a good thing to use the str() function
str(cheese)

# As we saw for the matrices, we can add/replace the names of the rows
rownames(cheese) <- c(
  "Camembert", 
  "Roquefort", 
  "Comté",
  "Pélardon", 
  "Tofu"
)

# Extract element from a dataframe
# first line of the dataframe
cheese[1,]
# last line
cheese[nrow(cheese),]
# Dimensions of a dataframe
dim(cheese)
nrow(cheese)
ncol(cheese)
# printing out the first line of a dataframe
head(cheese, 1) 
# printing out the last line of a dataframe
tail(cheese, 1)

# Select the lines corresponding to Roquefort & Pélardon
cheese[c(2, 4),] #using the indices
cheese[c("Roquefort", "Pélardon"),] #better if using directly the names

# Select the lines for Roquefort & Pélardon and Cooked & strength columns
cheese[c("Roquefort", "Pélardon"),c("Cooked", "strength")] #again using the names makes it easy

# Other option, using the subset function (see also dplyr::subset())
cheese_small <- base::subset(cheese, 
                             subset = c("Roquefort", "Pélardon") %in% rownames(cheese),
                             select = c("Cooked", "strength")
) 

# Boolean subsetting
# using columns containing booleans
cheese[cheese$Cooked,] # selects rows where the "Cooked" column is TRUE.

# Adding columns
cheese$isStrong <- cheese$strength>3

---

# ko vs wt analysis
# loading data
counts_wt1 <- read.table(file = "~/Documents/Teaching/Rcourse/April_2024/First_steps_RStudio/data/WT_1.tsv",
                         header = TRUE,
                         sep = "\t", 
                         stringsAsFactors = FALSE)
counts_wt2 <- read.table(file = "~/Documents/Teaching/Rcourse/April_2024/First_steps_RStudio/data/WT_2.tsv",
                         header = TRUE,
                         sep = "\t", 
                         stringsAsFactors = FALSE)

counts_ko1 <- read.table(file = "~/Documents/Teaching/Rcourse/April_2024/First_steps_RStudio/data/KO_1.tsv",
                         header = TRUE,
                         sep = "\t", 
                         stringsAsFactors = FALSE)

counts_ko2 <- read.table(file = "~/Documents/Teaching/Rcourse/April_2024/First_steps_RStudio/data/KO_2.tsv",
                         header = TRUE,
                         sep = "\t", 
                         stringsAsFactors = FALSE)

# Displaying the fisrt 10 lines of the dataframe counts_wt1
head(counts_wt1, 10)
# merging wt data
counts_allwt <- merge(x = counts_wt1, 
                      y = counts_wt2,
                      by = "gene_name",
                      all = FALSE)
head(counts_allwt)
nrow(counts_allwt)

# merging ko data
counts_allko <- merge(x = counts_ko1, 
                      y = counts_ko2,
                      by = "gene_name",
                      all = FALSE)
head(counts_allko)
nrow(counts_allko)

# merging all data (wt & ko)
counts_all <- merge(x = counts_allwt,
                    y = counts_allko,
                    by = "gene_name",
                    all = FALSE)
head(counts_all)
dim(counts_all)

# Normalizing data 
# Create a bar plot to visualize the total counts across all samples for each gene
barplot(colSums(counts_all[,-1]))
# Normalizing the data: Here columns are normalized by their column sums, to mitigate the difference in sequencing depth

# Step 1: Calculate the column sums of counts for each sample
col_sums <- colSums(counts_all[,-1])

# Step 2: Normalize each column by dividing each element by its corresponding column sum
#         Transpose the counts matrix to perform column-wise operations
#         Then transpose the result back to get back its original orientation
normalized_counts <- t(t(counts_all[,-1]) / col_sums)

# Step 3: Combine the normalized counts with the gene names
#         Use cbind to combine the gene names from the first column with the normalized counts
#         Create a new dataframe 'norm_data'
norm_data <- cbind("gene_name" = counts_all[,1], as.data.frame(normalized_counts))
# checking the created dataframe
dim(norm_data)
head(norm_data)
str(norm_data)

# Calculate the (norm) mean for each group
norm_data$mean_wt <- rowMeans(norm_data[,c("WT_1","WT_2")]) 
norm_data$mean_ko <- rowMeans(norm_data[,c("KO_1","KO_2")])
# Calculate the Fold Change
norm_data$FC <- norm_data$mean_ko / norm_data$mean_wt
# Log-transform the Fold Change
norm_data$logFC <- log2(norm_data$FC)
# Add "direction"
summary(norm_data$logFC)
table(abs(norm_data$logFC)>=.5)
# For this let's consider (arbitrarily) that 
# • the genes with a logFC<|0.5| are not differentially expressed, 
# • the genes with logFC >=0.5 are up regulated (compared to wt) and 
# • the genes with a logFC ≤ -0.5 are downregulated (compared to wt)

norm_data$dir <- ifelse(
  test = norm_data$logFC >= 0.5, # Step 1: Test if logFC is greater than or equal to 0.5.
  yes = "UP", # Step 1: If the test is true, assign "UP"
  no = ifelse( # Step 1: If the test is false, proceed to Step 2
    test = norm_data$logFC <= -0.5, # Step 2: Test if logFC is less than or equal to -0.5.
    yes = "DN", # Step 2: If the test is true, assign "DN" (downregulated)
    no = "not_deg" # Step 2: If the test is false, assign "not_deg" (not differentially expressed)
  )
)

# counting the different categories of genes
table(norm_data$dir)
# if we want to extract the name of all the down-regulated genes 
downgenes <- norm_data$gene_name[norm_data$dir == "DN"]
# showing the dataframe with only the up-regulated genes
norm_data[norm_data$dir == "UP",]

---

# iris subsetting
head(iris)
# Create a dataframe containing half of the population with the largest petal length
dim(iris)
summary(iris)
iris_large_petal <- iris[iris$Petal.Length >= median(iris$Petal.Length),] 

---

# Factors in mi dataset
# importing the mi dataset
mi <- read.csv(file = "~/Documents/Teaching/Rcourse/April_2024/First_steps_RStudio/data/mi.csv",stringsAsFactors = )
# check the structure of the Income column
str(mi$Income) #the default stringsAsFactors is FALSE so we have character strings
barplot(table(mi$Income)) #on the barplot, the different classes don't follow a logical order (see [0-1000] class position)
# Let's use factor to fix this
unique(mi$Income) #To get the different classes we want to reorder
mi$Income_f <- factor(
  mi$Income, 
  levels = c("[0-1000]", "(1000-2000]", "(2000-3000]", "(3000-inf]") #different classes in the desired order
)
str(mi$Income_f) #checking the nature of the variable
levels(mi$Income_f)
table(mi$Income, mi$Income_f) #checking we did not mess too much with the data
barplot(table(mi$Income_f)) #now the order of the classes is more logical

