# Lists
# Create a list
gene_info <- list("WBGene00004775", "sep-1", "I", 3433156, 3438543, FALSE)

# Extract element from a list
# Extract the 3rd element
gene_info[3] # this extract a list not the 3rd element of the list gene_info
str(gene_info[3]) 

gene_info[[3]] #This is the right command to extract the 3rd element of the list
str(gene_info[[3]])

# Name the elements of the list
names(gene_info) <- c("id", "name", "chrom", "start", "end", "fwd")

# Extract an element of the list using the names
gene_info$chrom
str(gene_info$chrom)
gene_info[["chrom"]]
gene_info["chrom"] #!!

# Add an element to a list
gene_info$example <- sample(letters, size = 15, replace = TRUE)

# Extract the 10th element of the vector example in the gene_info list
gene_info$example[10]

# List containing the 1st element and the last element of the list gene_info
length(gene_info)

# Create a shorter list containing the 1st and last elements of gene_info
gene_info_short <- gene_info[c(1, length(gene_info))]

# An alternative way to create the shorter list
gene_info_short2 <- list(gene_info[[1]], gene_info[[length(gene_info)]])
names(gene_info_short2) <- c(names(gene_info)[1], names(gene_info)[length(gene_info)])

# Display the shorter lists
gene_info_short
# Command below doesn't do want we wanted
gene_info_short3 <- list(c(gene_info[1], gene_info[length(gene_info)]))
gene_info_short3
#Compare the structure of the gene_info_short (what we wanted) and gene_info_short3
str(gene_info_short)
str(gene_info_short3)

# Add an additional element to gene_info
gene_info_exp <- c(gene_info, "elementxxx" = rnorm(n = 21))

# Add a list to a list (here we just duplicate gene_info)
gene_info_dup <- c(gene_info, gene_info)

# Small exercise on lists.
# Create a list containing 3 elements,
# each element is a vector containing random integers between -100 and 100,
# each vector should have a random size between 400 and 600.

# once it's done, give me
# a. the size of the list
# b. the size of each element of the list


my_list <- list(
  "elt1" = sample(x = -100 : 100, 
                  size = sample(x= 400:600, size = 1), 
                  replace = TRUE),
  "elt2" = sample(x = -100 : 100, 
                  size = sample(x= 400:600, size = 1), 
                  replace = TRUE),
  "elt3" = sample(x = -100 : 100, 
                  size = sample(x= 400:600, size = 1), 
                  replace = TRUE)
)

# Get the size of the list and each element
length(my_list)
str(my_list)
length(my_list[[1]])
length(my_list[[2]])
length(my_list[[3]])

# Loop through the list elements and print their lengths
for(i in 1:length(my_list)){
  cat("Length of the element", i, "=", length(my_list[[i]]), "\n")
}

# Using lapply to get the lengths of all elements
lapply(my_list, length)
lapply(my_list, function(x) length(x))

# Using sapply to get the lengths of all elements (does the same as lapply but the display is different)
sapply(my_list, length)
# Using lenghts (with a s!) to get the lengths of all elements
lengths(my_list)

# Add another element to my_list
my_list[["elt4"]] <- sample(x = -100 : 100, 
                            size = sample(x= 400:600, size = 1), 
                            replace = TRUE)

# Create a long list with nbOfElementTheBossWants elements
# 1.Create an empty list to store the elements
my_long_list <- list()

# 2.Define the number of elements the boss wants in the list
nbOfElementTheBossWants <- 20

# 3.Loop to generate random vectors and assign them to the list elements
for(i in 1:nbOfElementTheBossWants){
  # Generate a random vector of integers between -100 and 100
  # and assign it to the i-th element of the list
  my_long_list[[i]] <- sample(
    # Specify the range of integers to sample from (-100 to 100 inclusive)
    x = -100 : 100, 
    # Randomly choose the size of the vector between 400 and 600
    size = sample(
      # Specify the range of possible sizes (from 400 to 600)
      x = 400:600, 
      # Specify the number of samples to draw (in this case, just 1)
      size = 1
    ), 
    # Allow sampling with replacement (allowing the same integer to be chosen multiple times)
    replace = TRUE
  )
  # Assign a name to the current list element
  names(my_long_list)[i] <- paste("elt", i, sep = "_")
} #End of for loop.


# Get the count of negative values in each element of the long list
sapply(my_long_list, function(x) length(x[x<0]))

# Create a "positive" list containing only the values >0 (should be same size as original list)
my_long_pos_list <- lapply(my_long_list, function(x) return(x[x>0]))

# Check if negative values are present in the positive list
sapply(my_long_pos_list, function(x) 
  length(
    x[x<0] 
  )
)

