# Matrices
# Create a matrix from a vector
v1 <- c(4,8,43, 6,12,36, 11, 10,77) # create the vector
# Use the matrix() function to reshape the vector into a matrix
mat1 <- matrix(data = v1, nrow = 3, ncol = 3) #default option byrow = FALSE
mat2 <- matrix(data = v1, nrow = 3, ncol = 3, byrow = TRUE)

# Transposing a matrix
t_mat2 <- t(mat2)

# Dimensions of the matrix
dim(mat2)
nrow(mat2)
ncol(mat2)

# Extracting element(s) from a matrix
# Extract the second line from mat2
mat2[2,]
mat2[2] #don't forget the comma!

# Extract the third column
mat2[,3]

# Extract the third element of the third line
mat2[3,3]

# Extract the element of the last row and last column
mat2[nrow(mat2), ncol(mat2)]

# Adding names to the rows
rownames(mat2) <- c("Michel", "Jean-Paul", "Brigitte")

# Adding names to the columns
colnames(mat2) <- c("Day", "Month", "Year")

# We can use row/colname(s) to extract element(s) from a matrix
mat2["Michel", "Year"]
mat2[1,3] # Using indices still work

# Expand the matrix: Adding a row
mat2exp <- rbind(mat2,c(17, 3, 82))
# Adding a rowname afterwards
rownames(mat2exp)[length(rownames(mat2exp))] <- "Annie"
# Changing a rowname
rownames(mat2exp)[nrow(mat2exp)] <- "Jocelyne"

# Adding a row and the rowname at the same time
mat2exp <- rbind(mat2,"Lucy" = c(5, 8, 54)) #first way

Hugo <- c(1,1,22) # Second way the vector name will be used as rowname.
mat2exp <- rbind(mat2exp, Hugo) # Second way the vector name will be used as rowname.

# Adding columns
mat2expexp <- cbind(mat2exp, c(1:nrow(mat2exp)))

mat2expexp <- cbind(mat2expexp, 1:3) #Example of recycling

# ¡¡Matrices can only contain one type of variable!!
mat2expexp <- cbind(mat2expexp, letters[1:nrow(mat2expexp)])
# Difficult to correct this mistake, best solution is to go back and re-create the matrix
mat2expexp <- mat2expexp[,-ncol(mat2expexp)] #Removing the last column (the one containing the strings)
mat2expexp <- as.numeric(mat2expexp) #Force the variable to become numeric again... We loose the 2D-structure of the matrix!!

# Operation on Matrices (containing numbers)
mean(mat2expexp)
sum(mat2expexp)

# Create a new matrix
matexp <- matrix(0:15, nrow = 4)
# Calculate the percentage of zeros in the matrix
prop.table(table(matexp == 0))*100
# Logtransform the matrix (adding a 0.5 offset to avoid getting -Inf when logtransform zeros)
log_matexp <- log(matexp+0.5)

# Operation on the matrices and vectors
my_mat <- cbind(
  "s1" = 1:6,
  "s2" = c(2, 4, 6, 8, 10, 12),
  "s3" = c(3, 6, 9, 12, 15, 18))


# Operation on the matrices and vectors

# Create a matrix 'my_mat' using 'cbind', with three columns named 's1', 's2', and 's3'
# Each column contains a sequence of numbers from 1 to 6, multiples of 2, and multiples of 3, respectively
my_mat <- cbind(
  "s1" = 1:6,
  "s2" = c(2, 4, 6, 8, 10, 12),
  "s3" = c(3, 6, 9, 12, 15, 18))

# Assign row names to 'my_mat' 
rownames(my_mat) <- paste("gene", 1:nrow(my_mat), sep = "_")

# Display the matrix 'my_mat'
my_mat  

# Divide each element of 'my_mat' by the corresponding element of the vector 1:3
my_mat / 1:3
# how the operation is made? Row-wise or Column-wise ?
# -> Element-wise operations are applied row-wise by default
# Note also that the vector 1:3 is shorter than the number of rows in my_mat, so R recycles it to match the length of each row in my_mat.

# Calculate the row means of 'my_mat'
rowMeans(my_mat)

# Center 'my_mat' by subtracting the row means from each row
# As the operations are applied row-wise by default, substracting the the row means can be performed directly
my_mat_centered <- my_mat - rowMeans(my_mat)
my_mat_centered

# Display the updated matrix 'my_mat'
my_mat

# Now, what if we want to normalize the data by the sequencing depth (dividing each element by the corresponding column sum)
# Calculate the column sums of 'my_mat'
colSums(my_mat)

# Normalize 'my_mat' by dividing each element by the corresponding column sum
my_mat_norm <- my_mat / colSums(my_mat)
colSums(my_mat_norm)
# Does this work ? No, because the operation are applied row-wise and we want to the division to be applied column-wise

# Transpose 'my_mat' using 't', then normalize it again by dividing each element by the column sums of the transposed matrix
my_mat_norm <- t(t(my_mat) / colSums(my_mat_norm))
colSums(my_mat_norm)

# In conclusion: 
# Without transposition: Element-wise operations are applied row-wise by default. 
# If you want to apply the operation column-wise, you would need to transpose the matrix first. 
# Once the operation is done we transpose the normalized matrix again. This brings it back to its original orientation, with rows and columns restored to their original positions.

