# Function:Example.
# This function takes vector vec as input, identify outliers based on the Tukey method (1.5 times the IQR), and returns a corrected (i.e without outliers) vector .
remove_outlier <- function(vec = NULL) {
  # Check if 'vec' is NULL
  if (is.null(vec)) {
    # If 'vec' is NULL, print a message indicating that no input vector was provided
    cat("No input vector!!")
  } else {
    # If 'vec' is not NULL, proceed to remove outliers
    # Calculate the interquartile range (IQR) of the input vector
    iqr <- IQR(vec)
    # Calculate the first quartile (Q1) of the input vector
    Q1 <- quantile(vec)["25%"]
    # Calculate the third quartile (Q3) of the input vector
    Q3 <- quantile(vec)["75%"]
    # Calculate the lower bound for identifying outliers
    lower_ok <- Q1 - 1.5 * iqr
    # Calculate the upper bound for identifying outliers
    upper_ok <- Q3 + 1.5 * iqr
    # Use boolean test to identify which elements in 'vec' are not outliers
    not_outliers <- (lower_ok <= vec) & (vec <= upper_ok)
    # Extract the elements of 'vec' that are not outliers using the boolean subsetting
    vec_corrected <- vec[not_outliers]
    # Return the vector with outliers removed
    return(vec_corrected)
  }
}

---
  
# The syntax to create a function in R is as follows:
name_of_the_function <- function(){
  # Any commands or statements to be executed when the function is called
}
# It can be as simple as defining a function called 'sayHi' that prints "Hello World!" when called:
sayHi <- function(){
  print("Hello World!")
}
# to call the function, we just type its name followed by ()
sayHi()
# In this case, it prints the string "Hello World!" to the console
  
  
  